@@ -1,5 +1,48 @@
 var dt = {};
 
+var jinjafx = {
+  data: function(row, col) {
+    if ('_data' in jinjafx) {
+      if (typeof col === 'string') {
+        if (jinjafx._data[0].indexOf(col) !== -1) {
+          col = jinjafx._data[0].indexOf(col);
+        }
+        else {
+          console.log('invalid column "' + col + '" passed to jinjafx.data()');
+          return null;
+        }
+      }
+
+      if ((typeof row === 'number') && (typeof col === 'number')) {
+        if (jinjafx._data.length > row) {
+          if (jinjafx._data[row].length > col) {
+            return jinjafx._data[row][col];
+          }
+          else {
+            console.log('invalid column "' + col + '" passed to jinjafx.data()');
+            return null;
+          }
+        }
+        else {
+          console.log('invalid row "' + row + '" passed to jinjafx.data()');
+          return null;
+        }
+      }
+      else if (typeof row === 'number') {
+        if (jinjafx._data.length > row) {
+          return jinjafx._data[row];
+        }
+        else {
+          console.log('invalid row "' + row + '" passed to jinjafx.data()');
+          return null;
+        }
+      }
+    }
+    return null;
+  },
+  rows: 0
+};
+
 function reset_dt() {
   dt = {};
 }
@@ -142,10 +185,10 @@ function getStatusText(code) {
   };
 
   function select_dataset(e) {
-    switch_dataset(e.currentTarget.ds_name, true, false);
+    switch_dataset(e.currentTarget.ds_name, true/*, false*/);
   }
 
-  function switch_dataset(ds, sflag, dflag) {
+  function switch_dataset(ds, sflag/*, dflag*/) {
     if (sflag) {
       datasets[current_ds][0] = window.cmData.getDoc();
       datasets[current_ds][1] = window.cmVars.getDoc();
@@ -155,12 +198,12 @@ function getStatusText(code) {
       window.cmData.swapDoc(datasets[ds][0]);
       window.cmVars.swapDoc(datasets[ds][1]);
 
-      if (dflag) {
-        window.addEventListener('beforeunload', onBeforeUnload);
-        if (document.getElementById('get_link').value != 'false') {
-          document.title = 'JinjaFx [unsaved]';
-        }
+      // if (dflag) {
+      window.addEventListener('beforeunload', onBeforeUnload);
+      if (document.getElementById('get_link').value != 'false') {
+        document.title = 'JinjaFx [unsaved]';
       }
+      // }
       dirty = true;
       toggle_vault();
       document.getElementById('selected_ds').innerHTML = ds;
@@ -351,7 +394,7 @@ function getStatusText(code) {
   function delete_dataset(ds) {
     delete datasets[ds];
     rebuild_datasets();
-    switch_dataset(Object.keys(datasets).sort(default_on_top)[0], false, true);
+    switch_dataset(Object.keys(datasets).sort(default_on_top)[0], false/*, true*/);
     toggle_vault();
     fe.focus();
   }
@@ -431,7 +474,7 @@ function getStatusText(code) {
     }
   }
 
-  function jinjafx(method) {
+  function _jinjafx(method) {
     clear_status();
     fe.focus();
 
@@ -619,9 +662,9 @@ function getStatusText(code) {
                             document.getElementById('jinjafx_input_form').innerHTML = r_input_form;
                             input_form = vars['jinjafx_input']['body'];
 
-                            if (vars['jinjafx_input'].hasOwnProperty('script')) {
+                            if (input_script !== null) {
                               var script = document.createElement('script');
-                              script.innerHTML = vars['jinjafx_input']['script'];
+                              script.innerHTML = input_script;
                               document.getElementById('jinjafx_input').appendChild(script);
                             }
 
@@ -772,7 +815,7 @@ function getStatusText(code) {
             dt.global = e(window.cmgVars.getValue().replace(/\t/g, "  "));
           }
 
-          switch_dataset(current_ds, true, false);
+          switch_dataset(current_ds, true/*, false*/);
           Object.keys(datasets).sort(default_on_top).forEach(function(ds) {
             dt.datasets[ds] = {};
             dt.datasets[ds].data = e(datasets[ds][0].getValue());
@@ -1032,6 +1075,10 @@ function getStatusText(code) {
               }
 
               reset_location('/dt/' + dt_id);
+
+              if (qs.hasOwnProperty('g')) {
+                document.getElementById('generate').dispatchEvent(new Event('click'));
+              }
             }
             catch (e) {
               console.log(e);
@@ -1098,15 +1145,15 @@ function getStatusText(code) {
         xHR.send(null);
       }
   
-      document.getElementById('delete_ds').onclick = function() { jinjafx('delete_dataset'); };
-      document.getElementById('add_ds').onclick = function() { jinjafx('add_dataset'); };
-      document.getElementById('delete_t').onclick = function() { jinjafx('delete_template'); };
-      document.getElementById('add_t').onclick = function() { jinjafx('add_template'); };
-      document.getElementById('get').onclick = function() { jinjafx('get_link'); };
-      document.getElementById('get2').onclick = function() { jinjafx('get_link'); };
-      document.getElementById('update').onclick = function() { jinjafx('update_link'); };
-      document.getElementById('protect').onclick = function() { jinjafx('protect'); };
-      document.getElementById('delete').onclick = function() { jinjafx('delete'); };
+      document.getElementById('delete_ds').onclick = function() { _jinjafx('delete_dataset'); };
+      document.getElementById('add_ds').onclick = function() { _jinjafx('add_dataset'); };
+      document.getElementById('delete_t').onclick = function() { _jinjafx('delete_template'); };
+      document.getElementById('add_t').onclick = function() { _jinjafx('add_template'); };
+      document.getElementById('get').onclick = function() { _jinjafx('get_link'); };
+      document.getElementById('get2').onclick = function() { _jinjafx('get_link'); };
+      document.getElementById('update').onclick = function() { _jinjafx('update_link'); };
+      document.getElementById('protect').onclick = function() { _jinjafx('protect'); };
+      document.getElementById('delete').onclick = function() { _jinjafx('delete'); };
 
       if (window.crypto.subtle) {
         document.getElementById('encrypt').classList.remove('d-none');
@@ -1182,8 +1229,8 @@ function getStatusText(code) {
         }, false);
       }
   
-      document.getElementById('export').onclick = function() { jinjafx('export'); };
-      document.getElementById('generate').onclick = function() { jinjafx('generate'); };
+      document.getElementById('export').onclick = function() { _jinjafx('export'); };
+      document.getElementById('generate').onclick = function() { _jinjafx('generate'); };
       document.getElementById('encrypt').onclick = function() {
         clear_status();
         new bootstrap.Modal(document.getElementById('vault_encrypt'), {
@@ -1216,7 +1263,7 @@ function getStatusText(code) {
         },
         "Ctrl-S": function(cm) {
           if (!document.getElementById('update').classList.contains('d-none')) {
-            jinjafx('update_link');
+            _jinjafx('update_link');
           }
           else {
             set_status("darkred", "ERROR", "No Link to Update");
@@ -1224,17 +1271,17 @@ function getStatusText(code) {
         },
         "Cmd-S": function(cm) {
           if (!document.getElementById('update').classList.contains('d-none')) {
-            jinjafx('update_link');
+            _jinjafx('update_link');
           }
           else {
             set_status("darkred", "ERROR", "No Link to Update");
           }
         },
         "Ctrl-G": function(cm) {
-          jinjafx('generate');
+          _jinjafx('generate');
         },
         "Cmd-G": function(cm) {
-          jinjafx('generate');
+          _jinjafx('generate');
         },
         "Ctrl-D": false,
         "Cmd-D": false
@@ -1466,6 +1513,13 @@ function getStatusText(code) {
   
       document.getElementById('ml-input-reset').onclick = function(e) {
         document.getElementById('jinjafx_input_form').innerHTML = r_input_form;
+
+        if (input_script !== null) {
+          var script = document.createElement('script');
+          script.innerHTML = input_script;
+          document.getElementById('jinjafx_input').appendChild(script);
+        }
+
         var focusable = document.getElementById('jinjafx_input_form').querySelectorAll('input,select,textarea');
         if (focusable.length) {
           focusable[0].focus();
@@ -1485,12 +1539,26 @@ function getStatusText(code) {
                 if ((e.tagName == 'INPUT') && ((e.type == 'checkbox') || (e.type == 'radio'))) {
                   v = e.checked;
                 }
-                if (e.tagName == 'TEXTAREA') {
-                  vars[e.dataset.var] = '|2\n' + v.split(/\r?\n/g).map(function (e) {
-                    return '    ' + e;
-                  }).join('\r\n');
+                else if (e.tagName == 'SELECT') {
+                  var options = [];
+                  for (var i = 0; i < e.options.length; i++) {
+                    if (e.options[i].selected) {
+                      options.push(e.options[i].value || e.options[i].text);
+                    }
+                  }
+                  if (options.length) {
+                    if (vars.hasOwnProperty(e.dataset.var)) {
+                      vars[e.dataset.var] = vars[e.dataset.var].concat(options);
+                    }
+                    else {
+                      vars[e.dataset.var] = options;
+                    }
+                  }
                 }
                 else {
+                  if (e.tagName == 'TEXTAREA') {
+                    v = v.split(/\r?\n/g).join('\\n');
+                  }
                   if (vars.hasOwnProperty(e.dataset.var)) {
                     vars[e.dataset.var].push(v);
                   }
@@ -1518,7 +1586,7 @@ function getStatusText(code) {
               }
             }
             else {
-              vars_yml += '  ' + v + ': ' + vars[v];
+              vars_yml += '  ' + v + ': ' + vars[v] + '\r\n';
             }
           });
           dt.vars += '\r\n' + vars_yml;
@@ -1742,10 +1810,10 @@ function getStatusText(code) {
           if (existing == '') {
             datasets[new_ds] = [CodeMirror.Doc('', 'data'), CodeMirror.Doc('', 'yaml')];
             rebuild_datasets();
-            switch_dataset(new_ds, true, true);
+            switch_dataset(new_ds, true/*, true*/);
           }
           else {
-            switch_dataset(existing, true, true);
+            switch_dataset(existing, true/*, true*/);
           }
         }
         else {
@@ -2026,6 +2094,8 @@ function getStatusText(code) {
     var delim = new RegExp(cc > tc ? '[ \\t]*(?<!\\\\),[ \\t]*' : ' *\\t *');
     var hrow = datarows[0].split(delim);
 
+    jinjafx._data = [hrow];
+
     var table = '<table class="table table-responsive table-hover table-sm">';
     table += '<thead><tr>';
     for (var col = 0; col < hrow.length; col++) {
@@ -2036,6 +2106,8 @@ function getStatusText(code) {
     for (var row = 1; row < datarows.length; row++) {
       var rowdata = datarows[row].split(delim);
 
+      jinjafx._data.push(rowdata);
+
       if (rowdata.length != hrow.length) {
         table += '<tr class="table-danger">';
       }
@@ -2050,6 +2122,7 @@ function getStatusText(code) {
       table += '</tr>';
     }
     table += '</tbody></table>';
+    jinjafx.rows = datarows.length - 1;
     return table;
   }
 
@@ -2064,6 +2137,8 @@ function getStatusText(code) {
       csv_on = true;
     }
     else {
+      jinjafx.rows = 0;
+      delete jinjafx._data;
       window.cmData.getWrapperElement().style.display = 'block';
       document.getElementById("csv").style.display = 'none';
       window.cmData.refresh();
